﻿import com.delfiweb.error.*;

/*
@class 		 ReplaceString
@version 	 1.0
@author		 Cédric Néhémie
utilisation :  		 var myString:ReplaceString = new ReplaceString("my ReplaceString");
description :  Il s'agit de la classe String étendue avec des fonctions supplémentaires
*/
class com.delfiweb.utils.ReplaceString extends String
{	
	
	
	/*
	entrée :  	  string (String) la chaine pour initialiser la classe parente
	*/
	public function ReplaceString (string:String)
	{
		super (string);
	}
	
	
	
	/*
	entrée :  		  match (String) Le sample a recherché dans notre chaine
	entrée :  		  by (String) La chaine qui remplacera notre sample
	sortie :  	  String - Notre chaine transformée;
	utilisation :  		  var myNewString:ReplaceString = myString.strReplace(":lol:","<img src='lol.gif'/>");
	description :   Fait un "find and replace" dans la chaine.
	*/
	public function strReplace (match:String, by:String)
	{
		var pos:Number;
		var newstr:String = this;
		while (newstr.indexOf (match) != -1)
		{
			pos = newstr.indexOf (match);
			newstr = newstr.substr (0, pos) + by + newstr.substr (pos + match.length);
		}
		return new ReplaceString (newstr);
	}
	
	
	
	/*
	entrée :  		  match (Array) Le tableau de sample a recherché dans notre chaine
	entrée :  		  by (Array) Le tableau des chaines de remplacement
	sortie :  	  String - Notre chaine transformée;
	utilisation :  		  var myNewString:ReplaceString = myString.strReplace([":lol:",":)"],["<img src='lol.gif'/>","<img src='smile.gif'/>"]);
	description :   Fait un "find and replace" de plusieurs samples dans la chaine.
	*/
	public function arrayReplace (match:Array, by:Array):String
	{
		var m,b,newstr;
		
		m = match;
		b = by;
		newstr = this;
		
		while (m.length > 0)
		{
			newstr = newstr.split(m.pop()).join(b.pop());
		}
		
		return new ReplaceString (newstr);
	}
	
	
	
	/*
	entrée :  		  pattern (String) le tableau de chaine a recherché dans notre instance
	entrée :  		  rpattern (String) le tableau de chaine qui remplace nos pattern
	utilisation :  		  str.charReplace("éèà","eea");
	sortie :  	  ReplaceString - La nouvelle chaine
	@throw 		  Les deux chaines ne sont pas de la même taille.
	description :   permet de remplacer des caratères par d'autres.
	*/
	public function charReplace (pattern:String, rpattern:String):ReplaceString
	{
		try
		{
			if(pattern.length != rpattern.length) { throw new DError ("Les deux chaines ne sont pas de la même taille","ReplaceString.charReplace()"); }
			
			var str:ReplaceString = this;
			while(pattern.length > 0)
			{
				var char = pattern.substr(0,1);
				var rchar = rpattern.substr(0,1);
				str = str.strReplace(char,rchar);
				pattern = pattern.substr(1);
				rpattern = rpattern.substr(1);
			}			
			return new ReplaceString(str);
		}
		catch(e:_DError)
		{
			ErrorDispatcher.send(e);
		}
	}
	
	
	
	/*
	sortie :  	  String - La chaine transformée
	utilisation :  	 	  myString.UCFirst();
	description :   Met le premier caractère de la chaine en majuscule, comme son
				  équivalent en PHP
	*/
	public function UCFirst ():String
	{
		var nstr:String = this.substr (0, 1).toUpperCase () + this.substr (1);
		return new ReplaceString (nstr);
	}
	
	
	
	/*
	sortie :  	  String - La chaine transformée
	utilisation :  		  myString.UCWords();
	description :   Met le premier caractère de chaque mot de la chaine en majuscule, 
				  comme son équivalent en PHP.
	*/
	public function UCWords ():String
	{
		var a:Array = this.split(" ");
		for (var i in a)
		{
			var nstr:String = a[i].substr (0, 1).toUpperCase () + a[i].substr (1);
			a[i] = nstr;
		}
		return new ReplaceString (a.join(" "));
	}
	
	
	
	/*
	utilisation :  		  str.stringAsWordArray();
	sortie :  	  Array - le tableau de mots
	description :   Renvoi un tableau de tous les mots d'une chaine.
	*/
	public function stringAsWordArray ():Array
	{
		return this.split (" ");
	}
	
	
	
	/*
	utilisation :  		  str.urlToAnchor();
	sortie :  	  Array - la chaine complete
	description :   Convertit toutes les url de type http://* en lien html.
	*/
	public function urlToAnchor ():ReplaceString
	{
		var array:Array = this.stringAsWordArray ();
		for (var i in array)
		{
			if (array[i].indexOf ("http://")!= -1)
			{
				array[i] = "<a href='" + array[i] + "' target='_blank'>" + array[i] + "</a>";
			}
		}
		return new ReplaceString(array.join (" "));
	}
	
	
	
	/*
	utilisation :  		 str.removeHTML();
	sortie :  	 ReplaceString - La chaine nettoyé
	description :  Supprime toutes le balises HTML.
	*/
	public function removeHTML ():ReplaceString
	{
		var string:String = this;
		while (string.indexOf ("<") != -1 && string.indexOf (">") != -1)
		{
			var id1:Number = string.indexOf ("<");
			var id2:Number = string.indexOf (">");
			string = (id1!=0) ? string.substr (0, id1) + string.substr (id2+1) : string.substr (id2+1);
		}
		return new ReplaceString(string);
	}
	
	
	
	/*
	entrée :  		 exclude (Array) les balises a ignoré dans la suppression
	sortie :  	 (String) la chaine néttoyé des ses balises
	utilisation :  		 myString.stripTags(["b","a"]);
	description :  Supprime toutes les balises HTML ou XML d'une chaine
				 à l'exception des balises spécifiées en paramètres.
	*/
	public function stripTags (exclude:Array):String
	{
		if(exclude == undefined || exclude.length == 0)
		{
			var id1, id2, nstr;
			nstr = this;
			while (nstr.indexOf ("<") >= 0 && nstr.indexOf (">") >= 0)
			{
				id1 = nstr.indexOf ("<");
				id2 = nstr.indexOf (">");
				if (id1 != 0)
				{
					nstr = nstr.substr (0, id1) + nstr.substr (id2 + 1);
				}
				else
				{
					nstr = nstr.substr (id2 + 1);
				}
			}	
			sstr = nstr;
		}
		else
		{
			var id1, id2, nstr, sstr, tag;
			nstr = this;
			sstr = "";
			while (nstr.indexOf ("<") >= 0 && nstr.indexOf (">") >= 0)
			{
				id1 = nstr.indexOf ("<");
				id2 = nstr.indexOf (">");
				if (id1 != 0)
				{
					tag  = nstr.substr (id1 ,id2 + 1 - id1);
				}
				else
				{
					tag = nstr.substr (0,id2 + 1);
				}
				// si la balise est dans notre 
				if(compareTags(exclude,tag))
				{
					sstr += nstr.substr (0 ,id2 + 1);
					nstr = nstr.substr (id2 + 1);
				}
				else
				{
					sstr += nstr.substr (0 ,id1 );
					nstr = nstr.substr (id2 + 1);
				}
			}
		}
		return sstr;
	}
	
	
	private function compareTags(tags:Array,tag:String):Boolean
	{
		// on teste si le noeud a des attributs ou du moins un espace
		var tagname:String = ReplaceString.getTagName(tag);
		for(var i in tags)
		{
			if(tagname == tags[i]) { return true; }
		}
		return false;
	}
	
	
	
	/*	
	entrée :  		 string (String) La chaine d'ou extraire le nom du noeud
	utilisation :  		 ReplaceString.getTagName("<node>");
	sortie :  	 ReplaceString - Le nom du noeud
	description :  Analyse d'une chaine pour en retrouver un nom de noeud.
	*/
	public static function getTagName (str:String):String
	{
		try
		{
			var tag, c1, c2, c, vstr, id1, id2, id3;
			tag = "";
			c1 = str.indexOf ("<");
			c2 = str.indexOf ("</") ;
			c = (c2 != -1) ? c2 + 1 : c1;
			vstr = str.substr (c+1);
			id1 = vstr.indexOf (" ");
			id2 = vstr.indexOf (">");
			id3 = vstr.indexOf ("/>");
			switch (true)
			{
				case (id1 == -1 && id2 == -1 && id3 == -1) : 
					throw new DError("L'argument passé n'est pas une balises","ReplaceString.getTagName()");
					break;
				case (id3 != -1) : 
					tag = vstr.substr (0, id3);
					break;
				case (id1 > id2) :
					tag = vstr.substr (0, id2);
					break;
				default :
					tag = vstr.substr (0, id1);
					break;
			}
			return tag;
		}
		catch(e:_DError)
		{
			ErrorDispatcher.send(e);
		}
	}



	/**
	* utilisation : 		nstr = str.trim();
	* sortie :  		String 
	* description :   Renvoi la chaine débarassée des blanc du début et de la fin
	*/
	public function trim ():String
	{
		var str = this
		if(substr(0,1)==" ")
		{
			str = str.substr(1); 
		}
		if(substr(-1,1)==" ")
		{
			str = str.substr(0,-1); 
		}
		return str;
	}
	
	
	
	/**
	 *	utilisation : 		 nstr = str.HTMLEscape();
	 *	description :  Supprime les éléments < et > pour les remplacer par leur equivalent html
	 *  sortie : 		 La chaine depouillee de ses crochet
	 */
	public function HTMLEscape ():String
	{
		return arrayReplace(["<",">"],["&lt;","&gt;"])
	}
	
	
	
	
	/**
	 *	utilisation : 		 nstr = str.euroEscape();
	 *	description :  Remplace le code #euro; par € a cause du codage utf-8
	 *  sortie : 		 La chaîne avec les symboles €
	 */
	public function euroEscape ():String
	{
		return strReplace("#euro;","€")
	}
	
	public function numberOf(pattern:String)
	{
		var longeur=pattern.length
		var str=this
		var iDebut=0
		var nbr=0
		while((iDebut=this.indexOf(pattern,iDebut))!=-1)
		{
			nbr++
			iDebut=iDebut+longeur
		}
		return nbr
	}
	
	
}


